/*
** libgetopt.h for libcommon in /home/phil/Travail/Perso/libcommon/src/getopt
**
** Made by phil
** Login   <phil@fr.thalesgroup.com>
**
** Started on  sam. 04 juil. 2009 19:21:01 CEST phil
** Last update lun. 06 juil. 2009 21:05:38 CEST phil
*/

#ifndef LIBGETOPT_H_
# define LIBGETOPT_H_

#include "libutils.h"
#include "liblist.h"

typedef struct arglist	s_arglist;

/*!
** @brief This is a list used by the function my_getopt to store the
** argument's name and option.
**
** @arg arg is the argument's name.
** @arg next is the pointer on the next structure.
**
*/
struct		arglist
{
  char		*arg;
  s_arglist	*next;
};

s_arglist	*cm_getopt_opt_arg(const char	*opt);

int32_t		cm_getopt_opt_given(const char	*opt);

/*!
** @brief Generate a list of all found options using pattern and argv
**
** my_getopt is a useful getopt replacement. It return a list of all given
** options, each containing their eventual(s) argument(s).
** The argument pattern has the following structure :
** - each option is separated by ':'.
** - each option is completly written.
** - each option must be concatenated with a char representing the number of
** argument(s).
** - the above char must be :
**   ~ a digit to represent the number of args that must follow.
**   ~ a '+' to precise that the number of arg should be one or more.
**   ~ a '*' to precise that the number of arg should be zero or more.
**   ~ a '#' when the option has the following structure : --option=val.
** In the last two cases, all strings following the option in argv are added
** as option argument, while they do NOT begin with a '-'.
** Example :
** my_getopt(argc, argv, "-a0:-b1:--help0:-p+:--option#:-c3:-q*");
**
** @param argc from main
** @param argv from main
** @param pattern the option pattern, as explained above
**
** Here is an example :
**
** ./exec -b test -- file1 file2 file3
** ./exec -- file2
** ./exec -a foo --bar=2 -- file1 file2
** ...
**
** @return 0 if the given options correspond to the given pattern rules, or -1
**
** @author Philippe Thierry <philippe.THIERRY@fr.thalesgroup.com>
*/
int8_t		cm_getopt(int32_t		argc,
                          const char **		argv,
                          const char *  const	pattern);

#endif /* !LIBGETOPT_H_ */
