/*
** liblist.h for Project in trunk/src/lib/list
**
** Made by thierr_p
** Login   <philippe.THIERRY@fr.thalesgroup.com>
**
** Started on  sam 02 dc 2006 10:34:46 CET thierr_p
** Last update sam. 04 juil. 2009 18:54:17 CEST phil
*/
/*
** Copyright (C) 2009 - Philippe THIERRY
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#ifndef LIBLIST_H_
# define LIBLIST_H_

# define LISTEND	(-1)
# define LISTBEGIN	(0)

# include <stdlib.h>
# include <unistd.h>
# include <fcntl.h>

typedef struct cm_list	cm_list;

# include "libstring.h"
# include "libutils.h"

/*!
** @brief Generic list structure.
** Contains all tools function.
** Usage is recursive : add_elem of current elem add one just after it.
** del_elem purge current element.
** get_pos given current element position.
** Usage is :
** del_list(list)
** {
**   while (list->next)
**     del_list(list->next);
**   del_elem(list);
** }
** @arg elem the element to include.
** @arg add_elem function pointer to add an element just after the current
** one.
** @arg del_elem function pointer to del the current element.
** @arg get_pos function pointer returning the current element position.
** @arg free_elem function pointer which frees the current stored element.
** @arg print_elem function pointer which print the current element.
** @arg next pointer to the next element.
** @arg prev pointer to the previous element.
*/
struct		cm_list
{
  void		*elem;
  cm_list	*(*add_elem)(cm_list	*list,
                             void	*elem);
  cm_list	*(*del_elem)(cm_list	*list);
  uint32_t	(*get_pos)(cm_list	*list);
  void		(*free_elem)(void	*elem);
  void		(*print_elem)(void	*print);
  cm_list	*next;
  cm_list	*prev;
};

/*
** Type definition of configuration functions of the list (free_elem and
** print_elem)
*/
typedef void		(*ext_func)(void	*s);

/*
** This function initalize the generic list. An empty generic list contains
** one element, with its elem field to NULL.
** freed is pointed in each structure of the list and should free the elem
** field of the structure. If freed is NULL, elem will not be freed.
** print is pointed on each structure of the list, and print the elem field
** content. As elem is user-defined, print should be user-defined too. That's
** why this function should be passed as an argument. If not, the list can't
** be printed.
*/
cm_list			*cm_list_init(ext_func	freed,
                                      ext_func	print);

/*
** Print all the list. Call the user-defined print_elem of each element.
*/
void			cm_list_print(cm_list	*list);

void			cm_list_print_rev(cm_list	*list);

/*
** Purge all the list.
*/
void			cm_list_purge(cm_list	*list);

/*
** Del the given list element, calling the user-defined free_elem, and then
** destroying the curent cell.
*/
cm_list			*cm_list_elem_del(cm_list	*list);

/*
** add elem into the list. The element is added directly after the position of
** the 'list' argument.
*/
cm_list			*cm_list_elem_add(cm_list	*list,
                                          void		*elem);

/*
** Return the number of elements in the list.
*/
int32_t			cm_list_size(cm_list	*list);

/*
** Give the position in the list of the current element
*/
uint32_t		cm_list_position(cm_list	*list);


#endif /* !LIBLIST_H_ */
