/*
** memtrace.h for Project in /home/phil/Travail/Thales/hookconnector/nfhcctl/src/lib/memtrace
**
** Made by thierr_p
** Login   <philippe.THIERRY@fr.thalesgroup.com>
**
** Started on  mer 20 ao 2008 09:24:41 CET thierr_p
** Last update mar. 30 juin 2009 20:49:24 CEST phil
*/
/*
** Copyright (C) 2009 - Philippe THIERRY
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#ifndef LIBMEMTRACE_H_
# define LIBMEMTRACE_H_

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <ctype.h>
#include <limits.h>
#include <sys/types.h>
#include <inttypes.h>

#include "liblist.h"
#include "libutils.h"
#include "libstring.h"
#include "libdebug.h"

/****************************************************************************
** MEMTRACE interface
**
** This library allow dynamic memory allocation management.
** Its goal is to maintain information about allocated memory and
** make a proper cleaning when closing after a panic.
** If used (-DMEMTRACE option), xmalloc and xfree (defined in libutils)
** automaticly use this library. It is also possible to use directly the
** following functions.
**
****************************************************************************/


/*
** Release all dynamicly allocated block (free all managed pointers)
*/
int
cm_memtrace_release(void);

/*
** Return the head of the memtrace list.
*/
cm_list
*cm_memtrace_get_head(void);

/*
** Print the memtrace information (block address and size)
*/
void
cm_memtrace_print(void);

/*
** Called by xmalloc. Allocate a block of memory using malloc, check the
** non-NULL malloc return and manage the pointer in memtrace internal list
*/
void
*cm_memtrace_malloc(size_t	size);

/*
** Free a memtraced pointer. If no such pointer found, return a warning
** message.
*/
void
cm_memtrace_free(void	*p);

/*
** Return the total memory usage of the heap.
*/
int32_t
cm_memtrace_get_total_memory_usage(void);

/*
** To be used with memtrace_unlock. Free all allocated memory between
** memtrace_lock and memtrace_unlock. See the following example :
** [...]
** cm_memtrace_get_total_memory_usage()
** cm_memtrace_lock()
** foo = xmalloc(1024);
** bar = xmalloc(1024);
** cm_memtrace_get_total_memory_usage()
** cm_memtrace_unlock()
** cm_memtrace_get_total_memory_usage()
**
** Would gives :
** 48
** 2096
** 48
**
** memtrace_lock|unlock pair is recursive.
*/
void
cm_memtrace_lock(void);

/*
** Free all memory allocated since the last call to memtrace_lock. See above
** for more information.
*/
void cm_memtrace_unlock(void);

#endif /* !LIBMEMTRACE_H_ */
