/*
** my_strings.h for Project in /home/phil/Travail/Epita/parser/src
**
** Made by phil
** Login   <Philippe.THIERRY@fr.thalesgroup.com>
**
** Started on  lun 24 oct 2005 13:45:58 CEST phil
** Last update lun. 06 juil. 2009 21:11:54 CEST phil
*/
/*
** Copyright (C) 2009 - Philippe THIERRY
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#ifndef MY_STRINGS_H_
# define MY_STRINGS_H_

/*
** standard includes
*/
# include <stdlib.h>
# include <unistd.h>
# include <fcntl.h>
# include <sys/types.h>

# include "libutils.h"
# include "libdebug.h"

/****************************************************************************
** LIBSTRING interface
****************************************************************************/


/*!
** @brief Compare two unsigned char based strings
** \see libstring_ucompare.c
*/
int		cm_string_ustrcmp(const unsigned char	*s1,
                                  const unsigned char	*s2);

/*
** copy the s string in str. str must be allocated first.
** Return str.
*/
unsigned char	*cm_string_ustrcpy(unsigned char		*str,
                                   const unsigned char		*s);

/*
** Implode a string tab onto a unique string, using sepi as separator.
** The stab table must finish with a NULL field.
*/
char		*cm_string_implode(const char	**stab,
			           const char	*sepi);

/*
** Explode a string onto a string tab, using sep to detect separation.
** For example :
** my_explode("bbaccadd", 'a') generates the following table :
** [
**  "bb",
**  "cc",
**  "dd"
** ];
*/
char		**cm_string_explode(const char	*str,
			            char	sep);

/*
** Return the shortest prefix of str, using mark as end-of-refix marker.
** mark is not a content of the prefix.
*/
char		*cm_string_prefix(const char	*str,
			          char		mark);

/*
** Return the longest prefix of str using mark as marker.
*/
char		*cm_string_lprefix(const char	*str,
			           char		mark);

/*
** Return the shortest suffix of str using mark as marker.
*/
char		*cm_string_suffix(const char	*str,
			          char		mark);

/*
** Return the longest suffix of str, using mark as marker.
*/
char		*cm_string_lsuffix(const char	*str,
			           char		mark);

/*
** Upercase a string
*/
char		*cm_string_upercase(char		*s);

/*
** Lowercase a string
*/
char		*cm_string_downcase(char		*s);

/*
** Concat two paths generating a unix path.
*/
char		*pathcat(char	*str1,
			 char	*str2);

/*
** return a copy of the len first chars of str, on which suffix is added.
** For example :
** cm_string_reduce("aaaaaa", "...", 4);
** returns :
** "aaaa..."
*/
char		*cm_string_reduce(const char	*str,
			          const char	*suffix,
			          unsigned int	len);

/*
** count the number of occurencies of the char c in str.
*/
int		cm_string_occurency(const char		*str,
                                    const char		c);

#endif /* !MY_STRINGS_H_ */
